package org.blue.club.configuration;

import lombok.RequiredArgsConstructor;
import org.blue.club.annotation.Auth;
import org.blue.club.annotation.AuthDescription;
import org.blue.club.annotation.PermissionDescription;
import org.blue.club.dao.AuthDao;
import org.blue.club.dao.UserDao;
import org.blue.club.entities.dto.user.User;
import org.blue.club.entities.vo.data.UserVo;
import org.mmga.spring.boot.starter.componet.AuthorizationHandler;
import org.mmga.spring.boot.starter.componet.JwtUtils;
import org.mmga.spring.boot.starter.entities.Result;
import org.mmga.spring.boot.starter.exception.AuthorizationException;
import org.mmga.spring.boot.starter.utils.VoUtils;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

import java.lang.annotation.Annotation;
import java.util.Arrays;
import java.util.Optional;
import java.util.stream.Stream;

@Component
@RequiredArgsConstructor
public class CustomAuthorizationHandler implements AuthorizationHandler<User> {
    private final JwtUtils jwtUtils;
    private final UserDao userDao;
    private final VoUtils voUtils;
    private final AuthDao authDao;
    private User guestUser;

    @Override
    public Optional<User> auth(String token, Annotation ann) {
        if (ann instanceof Auth auth) {
            User user = guestUser;
            if (guestUser == null) {
                user = new User();
                user.setId(-1L);
                user.setName("游客");
                user.setAuth(authDao.selectOneWithRelationsById(1));
                guestUser = user;
            }
            if (token != null) {
                Optional<Long> i = jwtUtils.verifyToken(token);
                if (i.isEmpty()) throw new AuthorizationException(Result.failed(HttpStatus.UNAUTHORIZED, "token错误！"));
                Long userId = i.get();
                UserVo userVo = userDao.selectOneWithRelationsById(userId);
                if (userVo == null)
                    throw new AuthorizationException(Result.failed(HttpStatus.UNAUTHORIZED, "用户不存在！"));
                user = voUtils.vo2DtoSafe(userVo, User.class);
            }
            AuthDescription[] auths = auth.auths();
            Stream<AuthDescription> authStream = Arrays.stream(auths);
            boolean isAuthAccept = auths.length == 0 || (auth.authType().equals(Auth.LogicType.ANY) ? authStream.anyMatch(user::hasAuth) : authStream.allMatch(user::hasAuth));
            if (!isAuthAccept) throw new AuthorizationException(Result.failed(HttpStatus.UNAUTHORIZED, "权限不足！"));
            PermissionDescription[] permissions = auth.permissions();
            Stream<PermissionDescription> permissionStream = Arrays.stream(permissions);
            boolean isPermissionAccept = permissions.length == 0 || (auth.permissionType().equals(Auth.LogicType.ANY) ? permissionStream.anyMatch(user::hasPermission) : permissionStream.allMatch(user::hasPermission));
            if (!isPermissionAccept)
                throw new AuthorizationException(Result.failed(HttpStatus.UNAUTHORIZED, "权限不足！"));
            return Optional.of(user);
        }
        return Optional.empty();
    }
}
