package org.blue.club.entities.dto.user;

import com.mybatisflex.annotation.Id;
import com.mybatisflex.annotation.RelationManyToOne;
import jakarta.annotation.Nullable;
import lombok.Data;
import org.blue.club.annotation.AuthDescription;
import org.blue.club.annotation.DescriptionType;
import org.blue.club.annotation.PermissionDescription;
import org.blue.club.entities.vo.data.AuthVo;
import org.blue.club.entities.vo.data.ClubAuthVo;
import org.blue.club.entities.vo.data.ClubVo;
import org.mmga.spring.boot.starter.annotation.VoFieldMapper;
import org.mmga.spring.boot.starter.interfaces.IdHolder;

@Data
public class User implements IdHolder {
    @Id
    private Long id;
    private String name;
    @Nullable
    private String avatar;
    @RelationManyToOne(selfField = "auth", targetField = "id")
    @VoFieldMapper("auths")
    private AuthVo auth;
    @RelationManyToOne(joinTable = "club_user_auth", selfField = "id", joinSelfColumn = "user_id", targetField = "id", joinTargetColumn = "club_id")
    private ClubVo club;
    @RelationManyToOne(joinTable = "club_user_auth", selfField = "id", joinSelfColumn = "user_id", targetField = "id", joinTargetColumn = "auth_id")
    private ClubAuthVo clubAuth;

    public boolean hasPermission(long id) {
        return this.hasPermission(DescriptionType.NORMAL, id);
    }

    public boolean hasPermission(DescriptionType type, long id) {
        return (type.equals(DescriptionType.CLUB) ? clubAuth : auth).hasPermission(id);
    }

    public boolean hasPermission(PermissionDescription description) {
        return hasPermission(description.type(), description.value());
    }

    public boolean hasSuperAdminPermission() {
        return this.hasPermission(3);
    }

    public boolean hasAuth(AuthDescription description) {
        long authId = description.value();
        return (description.type().equals(DescriptionType.CLUB) ? clubAuth.getId() : auth.getId()) == authId;
    }
}
